@@ -61,6 +61,12 @@ Q = NOUNIQUE = _wcparse.NOUNIQUE
 
 K = MARK = 0x100000
 
+# Internal flags
+_EXTMATCHBASE = _wcparse._EXTMATCHBASE
+_RTL = _wcparse._RTL
+_NOABSOLUTE = _wcparse._NOABSOLUTE
+_PATHNAME = _wcparse.PATHNAME
+
 FLAG_MASK = (
     CASE |
     IGNORECASE |
@@ -81,8 +87,9 @@ FLAG_MASK = (
     FORCEUNIX |
     GLOBTILDE |
     NOUNIQUE |
-    _wcparse._RECURSIVEMATCH |
-    _wcparse._NOABSOLUTE
+    _EXTMATCHBASE |
+    _RTL |
+    _NOABSOLUTE
 )
 
 
@@ -90,19 +97,19 @@ def _flag_transform(flags):
     """Transform flags to glob defaults."""
 
     # Enabling both cancels out
-    if flags & _wcparse.FORCEUNIX and flags & _wcparse.FORCEWIN:
-        flags ^= _wcparse.FORCEWIN | _wcparse.FORCEUNIX
+    if flags & FORCEUNIX and flags & FORCEWIN:
+        flags ^= FORCEWIN | FORCEUNIX
 
     # Here we force `PATHNAME`.
-    flags = (flags & FLAG_MASK) | _wcparse.PATHNAME
-    if flags & _wcparse.REALPATH:
+    flags = (flags & FLAG_MASK) | _PATHNAME
+    if flags & REALPATH:
         if util.platform() == "windows":
-            if flags & _wcparse.FORCEUNIX:
-                flags ^= _wcparse.FORCEUNIX
-            flags |= _wcparse.FORCEWIN
+            if flags & FORCEUNIX:
+                flags ^= FORCEUNIX
+            flags |= FORCEWIN
         else:
-            if flags & _wcparse.FORCEWIN:
-                flags ^= _wcparse.FORCEWIN
+            if flags & FORCEWIN:
+                flags ^= FORCEWIN
 
     return flags
 
@@ -117,31 +124,34 @@ class Glob(object):
         self.is_bytes = isinstance(pattern[0], bytes)
         self.current = b'.' if self.is_bytes else '.'
         self.root_dir = util.fscodec(root_dir, self.is_bytes) if root_dir is not None else self.current
-        self.mark = bool(flags & MARK)
         self.nounique = bool(flags & NOUNIQUE)
+        self.mark = bool(flags & MARK)
         if self.mark:
             flags ^= MARK
         self.negateall = bool(flags & NEGATEALL)
         if self.negateall:
             flags ^= NEGATEALL
-        self.nodir = bool(flags & _wcparse.NODIR)
+        self.nodir = bool(flags & NODIR)
         if self.nodir:
-            flags ^= _wcparse.NODIR
-        self.flags = _flag_transform(flags | _wcparse.REALPATH)
+            flags ^= NODIR
+        # Right to left searching is only for matching
+        if flags & _RTL:  # pragma: no cover
+            flags ^= _RTL
+        self.flags = _flag_transform(flags | REALPATH)
         self.raw_chars = bool(self.flags & RAWCHARS)
         self.follow_links = bool(self.flags & FOLLOW)
         self.dot = bool(self.flags & DOTMATCH)
-        self.unix = not bool(self.flags & _wcparse.FORCEWIN)
+        self.unix = not bool(self.flags & FORCEWIN)
         self.negate = bool(self.flags & NEGATE)
-        self.globstar = bool(self.flags & _wcparse.GLOBSTAR)
-        self.braces = bool(self.flags & _wcparse.BRACE)
-        self.matchbase = bool(self.flags & _wcparse.MATCHBASE)
+        self.globstar = bool(self.flags & GLOBSTAR)
+        self.braces = bool(self.flags & BRACE)
+        self.matchbase = bool(self.flags & MATCHBASE)
         self.case_sensitive = _wcparse.get_case(self.flags)
         self.specials = (b'.', b'..') if self.is_bytes else ('.', '..')
         self.empty = b'' if self.is_bytes else ''
         self.limit = limit
         self._parse_patterns(pattern)
-        if self.flags & _wcparse.FORCEWIN:
+        if self.flags & FORCEWIN:
             self.sep = b'\\' if self.is_bytes else '\\'
         else:
             self.sep = b'/' if self.is_bytes else '/'
@@ -149,11 +159,10 @@ class Glob(object):
     def _iter_patterns(self, patterns):
         """Iterate expanded patterns."""
 
-        count = 1
         seen = set()
         for p in patterns:
             p = util.norm_pattern(p, not self.unix, self.raw_chars)
-            for expanded in _wcparse.expand(p, self.flags):
+            for count, expanded in enumerate(_wcparse.expand(p, self.flags), 1):
                 if 0 < self.limit < count:
                     raise _wcparse.PatternLimitException(
                         "Pattern limit exceeded the limit of {:d}".format(self.limit)
@@ -168,20 +177,18 @@ class Glob(object):
                     seen.add(expanded)
 
                 yield is_neg, expanded
-                count += 1
 
     def _parse_patterns(self, patterns):
         """Parse patterns."""
 
         self.pattern = []
         self.npatterns = []
-        nflags = self.flags | _wcparse.REALPATH
         for is_neg, p in self._iter_patterns(patterns):
             if is_neg:
                 # Treat the inverse pattern as a normal pattern if it matches, we will exclude.
                 # This is faster as compiled patterns usually compare the include patterns first,
                 # and then the exclude, but glob will already know it wants to include the file.
-                self.npatterns.append(_wcparse._compile(p, nflags))
+                self.npatterns.append(_wcparse._compile(p, self.flags))
             else:
                 self.pattern.append(_wcparse.WcPathSplit(p, self.flags).split())
 
@@ -194,7 +201,7 @@ class Glob(object):
 
         if self.nodir:
             ptype = _wcparse.BYTES if self.is_bytes else _wcparse.UNICODE
-            nodir = _wcparse.RE_WIN_NO_DIR[ptype] if self.flags & _wcparse.FORCEWIN else _wcparse.RE_NO_DIR[ptype]
+            nodir = _wcparse.RE_WIN_NO_DIR[ptype] if self.flags & FORCEWIN else _wcparse.RE_NO_DIR[ptype]
             self.npatterns.append(nodir)
 
         # A single positive pattern will not find multiples of the same file
@@ -278,7 +285,7 @@ class Glob(object):
 
         # Python will never return . or .., so fake it.
         for special in self.specials:
-            yield special, True, True
+            yield special, True, True, False
 
         try:
             if NO_SCANDIR_WORKAROUND:
@@ -298,10 +305,8 @@ class Glob(object):
                             else:
                                 # We don't care if a file is a link
                                 is_link = False
-                            if deep and not self.follow_links and is_link:
-                                continue
                             if (not dir_only or is_dir):
-                                yield f.name, is_dir, hidden
+                                yield f.name, is_dir, hidden, is_link
                         except OSError:  # pragma: no cover
                             pass
             else:
@@ -316,10 +321,8 @@ class Glob(object):
                         is_link = os.path.islink(path)
                     else:
                         is_link = False
-                    if deep and not self.follow_links and is_link:
-                        continue
                     if (not dir_only or is_dir):
-                        yield f, is_dir, hidden
+                        yield f, is_dir, hidden, is_link
 
         except OSError:  # pragma: no cover
             pass
@@ -328,17 +331,18 @@ class Glob(object):
         """Recursive directory glob."""
 
         files = list(self._iter(curdir, dir_only, deep))
-        for file, is_dir, hidden in files:
+        for file, is_dir, hidden, is_link in files:
             if file in self.specials:
                 if matcher is not None and matcher(file):
                     yield os.path.join(curdir, file), True
                 continue
 
             path = os.path.join(curdir, file)
-            if (matcher is None and not hidden) or (matcher and matcher(file)):
+            follow = not is_link or self.follow_links
+            if (matcher is None and not hidden and (follow or not deep)) or (matcher and matcher(file)):
                 yield path, is_dir
 
-            if deep and not hidden and is_dir:
+            if deep and not hidden and is_dir and follow:
                 yield from self._glob_dir(path, matcher, dir_only, deep)
 
     def _glob(self, curdir, this, rest):
@@ -430,7 +434,7 @@ class Glob(object):
             results = []
             matcher = self._get_matcher(curdir)
             files = list(self._iter(None, dir_only, False))
-            for file, is_dir, hidden in files:
+            for file, is_dir, hidden, is_link in files:
                 if file not in self.specials and (matcher is None or matcher(file)):
                     results.append((file, is_dir))
         else:
